@@ -229,15 +229,19 @@ netlink_parse_link(struct nlmsghdr *msg,
 		case IFLA_LINK:
 			/* Index of "lower" interface */
 			iff->lower_idx = *(int*)RTA_DATA(attribute);
+			log_debug("netlink", "attribute IFLA_LINK for %s: %d",
+			    iff->name ? iff->name : "(unknown)", iff->lower_idx);
+			break;
+		case IFLA_LINK_NETNSID:
+			/* Is the lower interface into another namesapce? */
+			iff->lower_idx = -1;
+			log_debug("netlink", "attribute IFLA_LINK_NETNSID received for %s",
+			    iff->name ? iff->name : "(unknown)");
 			break;
 		case IFLA_MASTER:
 			/* Index of master interface */
 			iff->upper_idx = *(int*)RTA_DATA(attribute);
 			break;
-		case IFLA_TXQLEN:
-			/* Transmit queue length */
-			iff->txqueue = *(int*)RTA_DATA(attribute);
-			break;
 		case IFLA_MTU:
 			/* Maximum Transmission Unit */
 			iff->mtu = *(int*)RTA_DATA(attribute);
@@ -326,6 +330,34 @@ netlink_parse_address(struct nlmsghdr *msg,
 	return 0;
 }
 
+/**
+ * Merge an old interface with a new one.
+ *
+ * Some properties may be absent in the new interface that should be copied over
+ * from the old one.
+ */
+void
+netlink_merge(struct interfaces_device *old, struct interfaces_device *new)
+{
+	if (new->alias == NULL) {
+		new->alias = old->alias;
+		old->alias = NULL;
+	}
+	if (new->address == NULL) {
+		new->address = old->address;
+		old->address = NULL;
+	}
+	if (new->mtu == 0)
+		new->mtu = old->mtu;
+	if (new->type == 0)
+		new->type = old->type;
+	if (new->vlanid == 0)
+		new->vlanid = old->vlanid;
+
+	/* It's not possible for lower link to change */
+	new->lower_idx = old->lower_idx;
+}
+
 /**
  * Receive netlink answer from the kernel.
  *
@@ -339,8 +371,9 @@ netlink_recv(int s,
     struct interfaces_device_list *ifs,
     struct interfaces_address_list *ifas)
 {
-	char reply[NETLINK_BUFFER] __attribute__ ((aligned));
-	int end = 0;
+	int end = 0, ret = 0;
+	int flags = MSG_PEEK | MSG_TRUNC;
+	struct iovec iov;
 	int link_update = 0;
 
 	struct interfaces_device *ifdold;
@@ -349,13 +382,16 @@ netlink_recv(int s,
 	struct interfaces_address *ifanew;
 	char addr[INET6_ADDRSTRLEN + 1];
 
+	iov.iov_len = NETLINK_BUFFER;
+	iov.iov_base = malloc(iov.iov_len);
+	if (!iov.iov_base) {
+		log_warn("netlink", "not enough memory");
+		return -1;
+	}
+
 	while (!end) {
 		ssize_t len;
 		struct nlmsghdr *msg;
-		struct iovec iov = {
-			.iov_base = reply,
-			.iov_len = NETLINK_BUFFER
-		};
 		struct sockaddr_nl peer = { .nl_family = AF_NETLINK };
 		struct msghdr rtnl_reply = {
 			.msg_iov = &iov,
@@ -364,17 +400,49 @@ netlink_recv(int s,
 			.msg_namelen = sizeof(struct sockaddr_nl)
 		};
 
-		len = recvmsg(s, &rtnl_reply, 0);
+retry:
+		len = recvmsg(s, &rtnl_reply, flags);
 		if (len == -1) {
 			if (errno == EAGAIN || errno == EWOULDBLOCK) {
 				log_debug("netlink", "should have received something, but didn't");
-				return 0;
+				ret = 0;
+				goto out;
 			}
-			log_warnx("netlink", "unable to receive netlink answer");
-			return -1;
+			log_warn("netlink", "unable to receive netlink answer");
+			ret = -1;
+			goto out;
 		}
-		if (!len) return 0;
-		for (msg = (struct nlmsghdr*)(void*)reply;
+		if (!len) {
+			ret = 0;
+			goto out;
+		}
+
+		if (iov.iov_len < len || (rtnl_reply.msg_flags & MSG_TRUNC)) {
+			void *tmp;
+
+			/* Provided buffer is not large enough, enlarge it
+			 * to size of len (which should be total length of the message)
+			 * and try again. */
+			iov.iov_len = len;
+			tmp = realloc(iov.iov_base, iov.iov_len);
+			if (!tmp) {
+				log_warn("netlink", "not enough memory");
+				ret = -1;
+				goto out;
+			}
+			log_debug("netlink", "enlarge message size to %zu bytes", len);
+			iov.iov_base = tmp;
+			flags = 0;
+			goto retry;
+		}
+
+		if (flags != 0) {
+			/* Buffer is big enough, do the actual reading */
+			flags = 0;
+			goto retry;
+		}
+
+		for (msg = (struct nlmsghdr*)(void*)(iov.iov_base);
 		     NLMSG_OK(msg, len);
 		     msg = NLMSG_NEXT(msg, len)) {
 			if (!(msg->nlmsg_flags & NLM_F_MULTI))
@@ -406,6 +474,7 @@ netlink_recv(int s,
 						} else {
 							log_debug("netlink", "interface %s/%s is updated",
 							    ifdold->name, ifdnew->name);
+							netlink_merge(ifdold, ifdnew);
 							TAILQ_INSERT_AFTER(ifs, ifdold, ifdnew, next);
 							TAILQ_REMOVE(ifs, ifdold, next);
 							interfaces_free_device(ifdold);
@@ -485,6 +554,7 @@ netlink_recv(int s,
 				    msg->nlmsg_type, msg->nlmsg_len);
 			}
 		}
+		flags = MSG_PEEK | MSG_TRUNC;
 	}
 end:
 	if (link_update) {
@@ -494,6 +564,9 @@ end:
 			if (iface1->upper_idx != -1 && iface1->upper_idx != iface1->index) {
 				TAILQ_FOREACH(iface2, ifs, next) {
 					if (iface1->upper_idx == iface2->index) {
+						log_debug("netlink",
+						    "upper interface for %s is %s",
+						    iface1->name, iface2->name);
 						iface1->upper = iface2;
 						break;
 					}
@@ -516,7 +589,15 @@ end:
 						 * loop. */
 						if (iface2->lower_idx == iface1->index) {
 							iface1->lower = NULL;
-						} else iface1->lower = iface2;
+							log_debug("netlink",
+							    "link loop detected between %s and %s",
+							    iface1->name, iface2->name);
+						} else {
+							log_debug("netlink",
+							    "lower interface for %s is %s",
+							    iface1->name, iface2->name);
+							iface1->lower = iface2;
+						}
 						break;
 					}
 					if (iface2 == NULL)
@@ -527,7 +608,10 @@ end:
 			}
 		}
 	}
-	return 0;
+
+out:
+	free(iov.iov_base);
+	return ret;
 }
 
 static int
